# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'create_model.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_CreateModel(object):
    def setupUi(self, CreateModel):
        CreateModel.setObjectName("CreateModel")
        CreateModel.resize(508, 537)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(CreateModel.sizePolicy().hasHeightForWidth())
        CreateModel.setSizePolicy(sizePolicy)
        CreateModel.setWizardStyle(QtWidgets.QWizard.ModernStyle)
        self.speciesPage = QtWidgets.QWizardPage()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.speciesPage.sizePolicy().hasHeightForWidth())
        self.speciesPage.setSizePolicy(sizePolicy)
        self.speciesPage.setObjectName("speciesPage")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.speciesPage)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.groupBox_2 = QtWidgets.QGroupBox(self.speciesPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.layout_a = QtWidgets.QVBoxLayout()
        self.layout_a.setObjectName("layout_a")
        self.verticalLayout_2.addLayout(self.layout_a)
        self.horizontalLayout.addWidget(self.groupBox_2)
        self.groupBox_1 = QtWidgets.QGroupBox(self.speciesPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_1.sizePolicy().hasHeightForWidth())
        self.groupBox_1.setSizePolicy(sizePolicy)
        self.groupBox_1.setObjectName("groupBox_1")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_1)
        self.verticalLayout.setObjectName("verticalLayout")
        self.layout_b = QtWidgets.QVBoxLayout()
        self.layout_b.setObjectName("layout_b")
        self.verticalLayout.addLayout(self.layout_b)
        self.horizontalLayout.addWidget(self.groupBox_1)
        CreateModel.addPage(self.speciesPage)
        self.spatialPage = QtWidgets.QWizardPage()
        self.spatialPage.setObjectName("spatialPage")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.spatialPage)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox = QtWidgets.QGroupBox(self.spatialPage)
        self.groupBox.setObjectName("groupBox")
        self.formLayout_2 = QtWidgets.QFormLayout(self.groupBox)
        self.formLayout_2.setObjectName("formLayout_2")
        self.gridWidthNumberOfAtomsLabel = QtWidgets.QLabel(self.groupBox)
        self.gridWidthNumberOfAtomsLabel.setObjectName("gridWidthNumberOfAtomsLabel")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.gridWidthNumberOfAtomsLabel)
        self.gridWidth = QtWidgets.QSpinBox(self.groupBox)
        self.gridWidth.setMaximum(100000000)
        self.gridWidth.setSingleStep(20)
        self.gridWidth.setProperty("value", 100)
        self.gridWidth.setObjectName("gridWidth")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.gridWidth)
        self.gridHeightNumberOfAtomsLabel = QtWidgets.QLabel(self.groupBox)
        self.gridHeightNumberOfAtomsLabel.setObjectName("gridHeightNumberOfAtomsLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.gridHeightNumberOfAtomsLabel)
        self.gridHeight = QtWidgets.QSpinBox(self.groupBox)
        self.gridHeight.setReadOnly(True)
        self.gridHeight.setMaximum(100000000)
        self.gridHeight.setSingleStep(20)
        self.gridHeight.setProperty("value", 100)
        self.gridHeight.setObjectName("gridHeight")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.gridHeight)
        self.keepRatioLabel = QtWidgets.QLabel(self.groupBox)
        self.keepRatioLabel.setObjectName("keepRatioLabel")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.keepRatioLabel)
        self.keepRatioCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.keepRatioCheckBox.setChecked(True)
        self.keepRatioCheckBox.setObjectName("keepRatioCheckBox")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.keepRatioCheckBox)
        self.distanceBetweenAtomsLabel = QtWidgets.QLabel(self.groupBox)
        self.distanceBetweenAtomsLabel.setObjectName("distanceBetweenAtomsLabel")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.distanceBetweenAtomsLabel)
        self.distanceBetweenAtoms = QtWidgets.QLineEdit(self.groupBox)
        self.distanceBetweenAtoms.setText("")
        self.distanceBetweenAtoms.setClearButtonEnabled(False)
        self.distanceBetweenAtoms.setObjectName("distanceBetweenAtoms")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.distanceBetweenAtoms)
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.label)
        self.label_atom_number = QtWidgets.QLabel(self.groupBox)
        self.label_atom_number.setObjectName("label_atom_number")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.label_atom_number)
        self.boxWidthLabel = QtWidgets.QLabel(self.groupBox)
        self.boxWidthLabel.setObjectName("boxWidthLabel")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.boxWidthLabel)
        self.boxWidthLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.boxWidthLineEdit.setObjectName("boxWidthLineEdit")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.boxWidthLineEdit)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.groupBox_4 = QtWidgets.QGroupBox(self.spatialPage)
        self.groupBox_4.setObjectName("groupBox_4")
        self.formLayout_3 = QtWidgets.QFormLayout(self.groupBox_4)
        self.formLayout_3.setObjectName("formLayout_3")
        self.xPeriodicBoundariesLabel = QtWidgets.QLabel(self.groupBox_4)
        self.xPeriodicBoundariesLabel.setObjectName("xPeriodicBoundariesLabel")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.xPeriodicBoundariesLabel)
        self.xPeriodicBoundariesCheckBox = QtWidgets.QCheckBox(self.groupBox_4)
        self.xPeriodicBoundariesCheckBox.setChecked(True)
        self.xPeriodicBoundariesCheckBox.setObjectName("xPeriodicBoundariesCheckBox")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.xPeriodicBoundariesCheckBox)
        self.yPeriodicBoudariesLabel = QtWidgets.QLabel(self.groupBox_4)
        self.yPeriodicBoudariesLabel.setObjectName("yPeriodicBoudariesLabel")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.yPeriodicBoudariesLabel)
        self.yPeriodicBoudariesCheckBox = QtWidgets.QCheckBox(self.groupBox_4)
        self.yPeriodicBoudariesCheckBox.setChecked(True)
        self.yPeriodicBoudariesCheckBox.setObjectName("yPeriodicBoudariesCheckBox")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.yPeriodicBoudariesCheckBox)
        self.verticalLayout_3.addWidget(self.groupBox_4)
        self.groupBox_3 = QtWidgets.QGroupBox(self.spatialPage)
        self.groupBox_3.setObjectName("groupBox_3")
        self.formLayout = QtWidgets.QFormLayout(self.groupBox_3)
        self.formLayout.setObjectName("formLayout")
        self.firstSpeciesMoleFractionLabel = QtWidgets.QLabel(self.groupBox_3)
        self.firstSpeciesMoleFractionLabel.setObjectName("firstSpeciesMoleFractionLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.firstSpeciesMoleFractionLabel)
        self.firstSpeciesMoleFraction = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.firstSpeciesMoleFraction.setMaximum(1.0)
        self.firstSpeciesMoleFraction.setSingleStep(0.1)
        self.firstSpeciesMoleFraction.setProperty("value", 1.0)
        self.firstSpeciesMoleFraction.setObjectName("firstSpeciesMoleFraction")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.firstSpeciesMoleFraction)
        self.verticalLayout_3.addWidget(self.groupBox_3)
        self.previewButton = QtWidgets.QPushButton(self.spatialPage)
        self.previewButton.setObjectName("previewButton")
        self.verticalLayout_3.addWidget(self.previewButton)
        CreateModel.addPage(self.spatialPage)
        self.otherPage = QtWidgets.QWizardPage()
        self.otherPage.setObjectName("otherPage")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.otherPage)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.groupBox_5 = QtWidgets.QGroupBox(self.otherPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_5.sizePolicy().hasHeightForWidth())
        self.groupBox_5.setSizePolicy(sizePolicy)
        self.groupBox_5.setObjectName("groupBox_5")
        self.formLayout_4 = QtWidgets.QFormLayout(self.groupBox_5)
        self.formLayout_4.setObjectName("formLayout_4")
        self.temperatureKLabel = QtWidgets.QLabel(self.groupBox_5)
        self.temperatureKLabel.setObjectName("temperatureKLabel")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.temperatureKLabel)
        self.temperatureKDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.groupBox_5)
        self.temperatureKDoubleSpinBox.setMinimum(0.0)
        self.temperatureKDoubleSpinBox.setMaximum(100000000000000.0)
        self.temperatureKDoubleSpinBox.setProperty("value", 1.0)
        self.temperatureKDoubleSpinBox.setObjectName("temperatureKDoubleSpinBox")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.temperatureKDoubleSpinBox)
        self.verticalLayout_4.addWidget(self.groupBox_5)
        self.groupBox_7 = QtWidgets.QGroupBox(self.otherPage)
        self.groupBox_7.setObjectName("groupBox_7")
        self.formLayout_6 = QtWidgets.QFormLayout(self.groupBox_7)
        self.formLayout_6.setObjectName("formLayout_6")
        self.timestepLabel = QtWidgets.QLabel(self.groupBox_7)
        self.timestepLabel.setObjectName("timestepLabel")
        self.formLayout_6.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.timestepLabel)
        self.timestepLineEdit = QtWidgets.QLineEdit(self.groupBox_7)
        self.timestepLineEdit.setObjectName("timestepLineEdit")
        self.formLayout_6.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.timestepLineEdit)
        self.resetTimestep = QtWidgets.QToolButton(self.groupBox_7)
        self.resetTimestep.setObjectName("resetTimestep")
        self.formLayout_6.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.resetTimestep)
        self.verticalLayout_4.addWidget(self.groupBox_7)
        self.groupBox_6 = QtWidgets.QGroupBox(self.otherPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_6.sizePolicy().hasHeightForWidth())
        self.groupBox_6.setSizePolicy(sizePolicy)
        self.groupBox_6.setObjectName("groupBox_6")
        self.formLayout_5 = QtWidgets.QFormLayout(self.groupBox_6)
        self.formLayout_5.setObjectName("formLayout_5")
        self.sigmaMLabel = QtWidgets.QLabel(self.groupBox_6)
        self.sigmaMLabel.setObjectName("sigmaMLabel")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.sigmaMLabel)
        self.sigmaMLineEdit = QtWidgets.QLineEdit(self.groupBox_6)
        self.sigmaMLineEdit.setObjectName("sigmaMLineEdit")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.sigmaMLineEdit)
        self.epsilonJLabel = QtWidgets.QLabel(self.groupBox_6)
        self.epsilonJLabel.setObjectName("epsilonJLabel")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.epsilonJLabel)
        self.epsilonJLineEdit = QtWidgets.QLineEdit(self.groupBox_6)
        self.epsilonJLineEdit.setObjectName("epsilonJLineEdit")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.epsilonJLineEdit)
        self.reset_ia_LJ = QtWidgets.QToolButton(self.groupBox_6)
        self.reset_ia_LJ.setObjectName("reset_ia_LJ")
        self.formLayout_5.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.reset_ia_LJ)
        self.verticalLayout_4.addWidget(self.groupBox_6)
        self.groupBox_8 = QtWidgets.QGroupBox(self.otherPage)
        self.groupBox_8.setObjectName("groupBox_8")
        self.formLayout_7 = QtWidgets.QFormLayout(self.groupBox_8)
        self.formLayout_7.setObjectName("formLayout_7")
        self.r_cut_aLabel = QtWidgets.QLabel(self.groupBox_8)
        self.r_cut_aLabel.setObjectName("r_cut_aLabel")
        self.formLayout_7.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.r_cut_aLabel)
        self.r_cut_aLineEdit = QtWidgets.QLineEdit(self.groupBox_8)
        self.r_cut_aLineEdit.setObjectName("r_cut_aLineEdit")
        self.formLayout_7.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.r_cut_aLineEdit)
        self.r_cut_bLabel = QtWidgets.QLabel(self.groupBox_8)
        self.r_cut_bLabel.setObjectName("r_cut_bLabel")
        self.formLayout_7.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.r_cut_bLabel)
        self.r_cut_bLineEdit = QtWidgets.QLineEdit(self.groupBox_8)
        self.r_cut_bLineEdit.setObjectName("r_cut_bLineEdit")
        self.formLayout_7.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.r_cut_bLineEdit)
        self.r_cut_abLabel = QtWidgets.QLabel(self.groupBox_8)
        self.r_cut_abLabel.setObjectName("r_cut_abLabel")
        self.formLayout_7.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.r_cut_abLabel)
        self.r_cut_abLineEdit = QtWidgets.QLineEdit(self.groupBox_8)
        self.r_cut_abLineEdit.setObjectName("r_cut_abLineEdit")
        self.formLayout_7.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.r_cut_abLineEdit)
        self.resetRcut = QtWidgets.QToolButton(self.groupBox_8)
        self.resetRcut.setObjectName("resetRcut")
        self.formLayout_7.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.resetRcut)
        self.verticalLayout_4.addWidget(self.groupBox_8)
        CreateModel.addPage(self.otherPage)
        self.gridWidthNumberOfAtomsLabel.setBuddy(self.gridWidth)
        self.gridHeightNumberOfAtomsLabel.setBuddy(self.gridHeight)
        self.distanceBetweenAtomsLabel.setBuddy(self.distanceBetweenAtoms)
        self.xPeriodicBoundariesLabel.setBuddy(self.xPeriodicBoundariesCheckBox)
        self.yPeriodicBoudariesLabel.setBuddy(self.yPeriodicBoudariesCheckBox)
        self.firstSpeciesMoleFractionLabel.setBuddy(self.firstSpeciesMoleFraction)
        self.temperatureKLabel.setBuddy(self.temperatureKDoubleSpinBox)
        self.timestepLabel.setBuddy(self.timestepLineEdit)
        self.sigmaMLabel.setBuddy(self.sigmaMLineEdit)
        self.epsilonJLabel.setBuddy(self.epsilonJLineEdit)

        self.retranslateUi(CreateModel)
        QtCore.QMetaObject.connectSlotsByName(CreateModel)
        CreateModel.setTabOrder(self.gridWidth, self.gridHeight)
        CreateModel.setTabOrder(self.gridHeight, self.keepRatioCheckBox)
        CreateModel.setTabOrder(self.keepRatioCheckBox, self.distanceBetweenAtoms)
        CreateModel.setTabOrder(self.distanceBetweenAtoms, self.boxWidthLineEdit)
        CreateModel.setTabOrder(self.boxWidthLineEdit, self.xPeriodicBoundariesCheckBox)
        CreateModel.setTabOrder(self.xPeriodicBoundariesCheckBox, self.yPeriodicBoudariesCheckBox)
        CreateModel.setTabOrder(self.yPeriodicBoudariesCheckBox, self.firstSpeciesMoleFraction)
        CreateModel.setTabOrder(self.firstSpeciesMoleFraction, self.previewButton)
        CreateModel.setTabOrder(self.previewButton, self.temperatureKDoubleSpinBox)
        CreateModel.setTabOrder(self.temperatureKDoubleSpinBox, self.timestepLineEdit)
        CreateModel.setTabOrder(self.timestepLineEdit, self.sigmaMLineEdit)
        CreateModel.setTabOrder(self.sigmaMLineEdit, self.epsilonJLineEdit)
        CreateModel.setTabOrder(self.epsilonJLineEdit, self.reset_ia_LJ)

    def retranslateUi(self, CreateModel):
        _translate = QtCore.QCoreApplication.translate
        CreateModel.setWindowTitle(_translate("CreateModel", "Create Model"))
        self.speciesPage.setTitle(_translate("CreateModel", "Species definition"))
        self.groupBox_2.setTitle(_translate("CreateModel", "First species"))
        self.groupBox_1.setTitle(_translate("CreateModel", "Second species"))
        self.spatialPage.setTitle(_translate("CreateModel", "Spatial configuration"))
        self.groupBox.setTitle(_translate("CreateModel", "Grid"))
        self.gridWidthNumberOfAtomsLabel.setText(_translate("CreateModel", "Grid width (number of atoms)"))
        self.gridHeightNumberOfAtomsLabel.setText(_translate("CreateModel", "Grid height (number of atoms)"))
        self.keepRatioLabel.setText(_translate("CreateModel", "Keep equal ratio"))
        self.distanceBetweenAtomsLabel.setText(_translate("CreateModel", "Distance between atoms (m)"))
        self.label.setText(_translate("CreateModel", "Atom number"))
        self.label_atom_number.setText(_translate("CreateModel", "10000"))
        self.boxWidthLabel.setText(_translate("CreateModel", "Box width (m)"))
        self.groupBox_4.setTitle(_translate("CreateModel", "Boundaries"))
        self.xPeriodicBoundariesLabel.setText(_translate("CreateModel", "Enable X periodic boundaries"))
        self.yPeriodicBoudariesLabel.setText(_translate("CreateModel", "Enable Y periodic boundaries"))
        self.groupBox_3.setTitle(_translate("CreateModel", "Mixup"))
        self.firstSpeciesMoleFractionLabel.setText(_translate("CreateModel", "First species mole fraction"))
        self.previewButton.setText(_translate("CreateModel", "Preview"))
        self.otherPage.setTitle(_translate("CreateModel", "Other parameters"))
        self.groupBox_5.setTitle(_translate("CreateModel", "Thermodynamics"))
        self.temperatureKLabel.setText(_translate("CreateModel", "Temperature (K)"))
        self.groupBox_7.setTitle(_translate("CreateModel", "Simulation"))
        self.timestepLabel.setText(_translate("CreateModel", "Timestep (s)"))
        self.resetTimestep.setText(_translate("CreateModel", "Reset"))
        self.groupBox_6.setTitle(_translate("CreateModel", "Inter-atomic Lennard-Jones parameters"))
        self.sigmaMLabel.setText(_translate("CreateModel", "Sigma (m)"))
        self.epsilonJLabel.setText(_translate("CreateModel", "Epsilon (J)"))
        self.reset_ia_LJ.setText(_translate("CreateModel", "Reset"))
        self.groupBox_8.setTitle(_translate("CreateModel", "Performance"))
        self.r_cut_aLabel.setText(_translate("CreateModel", "r_cut_a (m)"))
        self.r_cut_bLabel.setText(_translate("CreateModel", "r_cut_b (m)"))
        self.r_cut_abLabel.setText(_translate("CreateModel", "r_cut_ab (m)"))
        self.resetRcut.setText(_translate("CreateModel", "Reset"))

