# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(668, 701)
        MainWindow.setWindowTitle("OpenMoldyn")
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_model = QtWidgets.QWidget()
        self.tab_model.setObjectName("tab_model")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab_model)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.groupBox = QtWidgets.QGroupBox(self.tab_model)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.paramsTreeWidget = QtWidgets.QTreeWidget(self.groupBox)
        self.paramsTreeWidget.setObjectName("paramsTreeWidget")
        self.horizontalLayout.addWidget(self.paramsTreeWidget)
        self.gridLayout_2.addWidget(self.groupBox, 3, 0, 1, 8)
        self.loadSimuBtn = QtWidgets.QToolButton(self.tab_model)
        self.loadSimuBtn.setObjectName("loadSimuBtn")
        self.gridLayout_2.addWidget(self.loadSimuBtn, 1, 7, 1, 1)
        self.gotoSimuBtn = QtWidgets.QCommandLinkButton(self.tab_model)
        self.gotoSimuBtn.setEnabled(False)
        self.gotoSimuBtn.setObjectName("gotoSimuBtn")
        self.gridLayout_2.addWidget(self.gotoSimuBtn, 5, 0, 1, 8)
        self.loadModelBtn = QtWidgets.QToolButton(self.tab_model)
        self.loadModelBtn.setObjectName("loadModelBtn")
        self.gridLayout_2.addWidget(self.loadModelBtn, 1, 1, 1, 1)
        self.newModelBtn = QtWidgets.QToolButton(self.tab_model)
        self.newModelBtn.setObjectName("newModelBtn")
        self.gridLayout_2.addWidget(self.newModelBtn, 1, 0, 1, 1)
        self.saveModelBtn = QtWidgets.QToolButton(self.tab_model)
        self.saveModelBtn.setEnabled(False)
        self.saveModelBtn.setObjectName("saveModelBtn")
        self.gridLayout_2.addWidget(self.saveModelBtn, 1, 2, 1, 1)
        self.newSimuBtn = QtWidgets.QToolButton(self.tab_model)
        self.newSimuBtn.setEnabled(False)
        self.newSimuBtn.setObjectName("newSimuBtn")
        self.gridLayout_2.addWidget(self.newSimuBtn, 1, 5, 1, 1)
        self.tabWidget.addTab(self.tab_model, "")
        self.tab_simu = QtWidgets.QWidget()
        self.tab_simu.setEnabled(False)
        self.tab_simu.setObjectName("tab_simu")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.tab_simu)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.progress_groupBox = QtWidgets.QGroupBox(self.tab_simu)
        self.progress_groupBox.setObjectName("progress_groupBox")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.progress_groupBox)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.ETA = QtWidgets.QLabel(self.progress_groupBox)
        self.ETA.setObjectName("ETA")
        self.gridLayout_4.addWidget(self.ETA, 0, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.progress_groupBox)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem, 0, 2, 1, 1)
        self.simuProgressBar = QtWidgets.QProgressBar(self.progress_groupBox)
        self.simuProgressBar.setMaximum(1)
        self.simuProgressBar.setProperty("value", 0)
        self.simuProgressBar.setObjectName("simuProgressBar")
        self.gridLayout_4.addWidget(self.simuProgressBar, 1, 0, 1, 3)
        self.gridLayout_3.addWidget(self.progress_groupBox, 4, 0, 1, 2)
        self.simuBtn = QtWidgets.QCommandLinkButton(self.tab_simu)
        self.simuBtn.setObjectName("simuBtn")
        self.gridLayout_3.addWidget(self.simuBtn, 1, 1, 1, 1)
        self.gotoProcessBtn = QtWidgets.QCommandLinkButton(self.tab_simu)
        self.gotoProcessBtn.setEnabled(False)
        self.gotoProcessBtn.setObjectName("gotoProcessBtn")
        self.gridLayout_3.addWidget(self.gotoProcessBtn, 5, 0, 1, 2)
        self.simuParams = QtWidgets.QFormLayout()
        self.simuParams.setObjectName("simuParams")
        self.iterationsLabel = QtWidgets.QLabel(self.tab_simu)
        self.iterationsLabel.setObjectName("iterationsLabel")
        self.simuParams.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.iterationsLabel)
        self.iterationsSpinBox = QtWidgets.QSpinBox(self.tab_simu)
        self.iterationsSpinBox.setMaximum(210000766)
        self.iterationsSpinBox.setSingleStep(100)
        self.iterationsSpinBox.setProperty("value", 1000)
        self.iterationsSpinBox.setObjectName("iterationsSpinBox")
        self.simuParams.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.iterationsSpinBox)
        self.simulationTimeLabel = QtWidgets.QLabel(self.tab_simu)
        self.simulationTimeLabel.setObjectName("simulationTimeLabel")
        self.simuParams.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.simulationTimeLabel)
        self.simulationTimeLineEdit = QtWidgets.QLineEdit(self.tab_simu)
        self.simulationTimeLineEdit.setObjectName("simulationTimeLineEdit")
        self.simuParams.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.simulationTimeLineEdit)
        self.saveAllAtomsPositionLabel = QtWidgets.QLabel(self.tab_simu)
        self.saveAllAtomsPositionLabel.setObjectName("saveAllAtomsPositionLabel")
        self.simuParams.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.saveAllAtomsPositionLabel)
        self.saveAllAtomsPositionCheckBox = QtWidgets.QCheckBox(self.tab_simu)
        self.saveAllAtomsPositionCheckBox.setObjectName("saveAllAtomsPositionCheckBox")
        self.simuParams.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.saveAllAtomsPositionCheckBox)
        self.tryToUseGPULabel = QtWidgets.QLabel(self.tab_simu)
        self.tryToUseGPULabel.setObjectName("tryToUseGPULabel")
        self.simuParams.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.tryToUseGPULabel)
        self.tryToUseGPUCheckBox = QtWidgets.QCheckBox(self.tab_simu)
        self.tryToUseGPUCheckBox.setChecked(True)
        self.tryToUseGPUCheckBox.setTristate(False)
        self.tryToUseGPUCheckBox.setObjectName("tryToUseGPUCheckBox")
        self.simuParams.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.tryToUseGPUCheckBox)
        self.gridLayout_3.addLayout(self.simuParams, 1, 0, 1, 1)
        self.temperature_groupBox = QtWidgets.QGroupBox(self.tab_simu)
        self.temperature_groupBox.setObjectName("temperature_groupBox")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.temperature_groupBox)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.designTemperatureBtn = QtWidgets.QToolButton(self.temperature_groupBox)
        self.designTemperatureBtn.setObjectName("designTemperatureBtn")
        self.gridLayout_9.addWidget(self.designTemperatureBtn, 0, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(522, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_9.addItem(spacerItem1, 0, 2, 1, 1)
        self.editModelBtn = QtWidgets.QToolButton(self.temperature_groupBox)
        self.editModelBtn.setEnabled(False)
        self.editModelBtn.setObjectName("editModelBtn")
        self.gridLayout_9.addWidget(self.editModelBtn, 0, 1, 1, 1)
        self.gridLayout_3.addWidget(self.temperature_groupBox, 2, 0, 1, 2)
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab_simu)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_6.setObjectName("gridLayout_6")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_6.addItem(spacerItem2, 1, 1, 1, 1)
        self.RTViewBtn = QtWidgets.QToolButton(self.groupBox_3)
        self.RTViewBtn.setObjectName("RTViewBtn")
        self.gridLayout_6.addWidget(self.RTViewBtn, 1, 2, 1, 1)
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.currentIterationLabel = QtWidgets.QLabel(self.groupBox_3)
        self.currentIterationLabel.setObjectName("currentIterationLabel")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.currentIterationLabel)
        self.currentIteration = QtWidgets.QLabel(self.groupBox_3)
        self.currentIteration.setObjectName("currentIteration")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.currentIteration)
        self.label_4 = QtWidgets.QLabel(self.groupBox_3)
        self.label_4.setObjectName("label_4")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.currentTime = QtWidgets.QLabel(self.groupBox_3)
        self.currentTime.setObjectName("currentTime")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.currentTime)
        self.gridLayout_6.addLayout(self.formLayout_2, 1, 0, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox_3, 3, 0, 1, 2)
        self.tabWidget.addTab(self.tab_simu, "")
        self.tab_processing = QtWidgets.QWidget()
        self.tab_processing.setEnabled(False)
        self.tab_processing.setObjectName("tab_processing")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.tab_processing)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.groupBox1d = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox1d.setObjectName("groupBox1d")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox1d)
        self.gridLayout.setObjectName("gridLayout")
        self.lineListW = QtWidgets.QListWidget(self.groupBox1d)
        self.lineListW.setObjectName("lineListW")
        self.gridLayout.addWidget(self.lineListW, 0, 0, 2, 1)
        self.formLayout_3 = QtWidgets.QFormLayout()
        self.formLayout_3.setObjectName("formLayout_3")
        self.asAFunctionOfLabel = QtWidgets.QLabel(self.groupBox1d)
        self.asAFunctionOfLabel.setObjectName("asAFunctionOfLabel")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.asAFunctionOfLabel)
        self.lineComboW = QtWidgets.QComboBox(self.groupBox1d)
        self.lineComboW.setObjectName("lineComboW")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lineComboW)
        self.plotB = QtWidgets.QPushButton(self.groupBox1d)
        self.plotB.setObjectName("plotB")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.plotB)
        self.gridLayout.addLayout(self.formLayout_3, 0, 1, 1, 1)
        self.gridLayout_5.addWidget(self.groupBox1d, 2, 0, 1, 1)
        self.groupBox_1doptions = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox_1doptions.setObjectName("groupBox_1doptions")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.groupBox_1doptions)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.list1DOptions = QtWidgets.QListWidget(self.groupBox_1doptions)
        self.list1DOptions.setObjectName("list1DOptions")
        self.horizontalLayout_4.addWidget(self.list1DOptions)
        self.gridLayout_5.addWidget(self.groupBox_1doptions, 2, 1, 1, 1)
        self.groupBoxMovie = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBoxMovie.setObjectName("groupBoxMovie")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.groupBoxMovie)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.makeMovieBtn = QtWidgets.QToolButton(self.groupBoxMovie)
        self.makeMovieBtn.setObjectName("makeMovieBtn")
        self.gridLayout_8.addWidget(self.makeMovieBtn, 0, 2, 1, 1)
        self.formLayout_4 = QtWidgets.QFormLayout()
        self.formLayout_4.setObjectName("formLayout_4")
        self.fPSLabel = QtWidgets.QLabel(self.groupBoxMovie)
        self.fPSLabel.setObjectName("fPSLabel")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.fPSLabel)
        self.FPSSpinBox = QtWidgets.QSpinBox(self.groupBoxMovie)
        self.FPSSpinBox.setProperty("value", 24)
        self.FPSSpinBox.setObjectName("FPSSpinBox")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.FPSSpinBox)
        self.stepsByFrameLabel = QtWidgets.QLabel(self.groupBoxMovie)
        self.stepsByFrameLabel.setObjectName("stepsByFrameLabel")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.stepsByFrameLabel)
        self.stepsByFrameSpinBox = QtWidgets.QSpinBox(self.groupBoxMovie)
        self.stepsByFrameSpinBox.setProperty("value", 10)
        self.stepsByFrameSpinBox.setObjectName("stepsByFrameSpinBox")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.stepsByFrameSpinBox)
        self.gridLayout_8.addLayout(self.formLayout_4, 0, 0, 1, 1)
        self.movieProgressBar = QtWidgets.QProgressBar(self.groupBoxMovie)
        self.movieProgressBar.setProperty("value", 0)
        self.movieProgressBar.setObjectName("movieProgressBar")
        self.gridLayout_8.addWidget(self.movieProgressBar, 0, 1, 1, 1)
        self.gridLayout_5.addWidget(self.groupBoxMovie, 5, 0, 1, 2)
        self.groupBox1d_stat = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox1d_stat.setObjectName("groupBox1d_stat")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.groupBox1d_stat)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.PDFButton = QtWidgets.QToolButton(self.groupBox1d_stat)
        self.PDFButton.setObjectName("PDFButton")
        self.horizontalLayout_5.addWidget(self.PDFButton)
        self.label = QtWidgets.QLabel(self.groupBox1d_stat)
        self.label.setObjectName("label")
        self.horizontalLayout_5.addWidget(self.label)
        self.PDFNSpinBox = QtWidgets.QSpinBox(self.groupBox1d_stat)
        self.PDFNSpinBox.setMinimum(1)
        self.PDFNSpinBox.setMaximum(100000)
        self.PDFNSpinBox.setSingleStep(100)
        self.PDFNSpinBox.setProperty("value", 1000)
        self.PDFNSpinBox.setObjectName("PDFNSpinBox")
        self.horizontalLayout_5.addWidget(self.PDFNSpinBox)
        self.label_2 = QtWidgets.QLabel(self.groupBox1d_stat)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_5.addWidget(self.label_2)
        self.PDFDistSpinBox = QtWidgets.QDoubleSpinBox(self.groupBox1d_stat)
        self.PDFDistSpinBox.setSingleStep(0.1)
        self.PDFDistSpinBox.setProperty("value", 1.5)
        self.PDFDistSpinBox.setObjectName("PDFDistSpinBox")
        self.horizontalLayout_5.addWidget(self.PDFDistSpinBox)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem3)
        self.gridLayout_5.addWidget(self.groupBox1d_stat, 1, 0, 1, 2)
        self.groupBox_data_mgt = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox_data_mgt.setObjectName("groupBox_data_mgt")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBox_data_mgt)
        self.gridLayout_7.setObjectName("gridLayout_7")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem4, 0, 3, 1, 1)
        self.saveSimuBtn = QtWidgets.QToolButton(self.groupBox_data_mgt)
        self.saveSimuBtn.setObjectName("saveSimuBtn")
        self.gridLayout_7.addWidget(self.saveSimuBtn, 0, 2, 1, 1)
        self.saveRModelBtn = QtWidgets.QToolButton(self.groupBox_data_mgt)
        self.saveRModelBtn.setObjectName("saveRModelBtn")
        self.gridLayout_7.addWidget(self.saveRModelBtn, 0, 0, 1, 1)
        self.reuseModelBtn = QtWidgets.QToolButton(self.groupBox_data_mgt)
        self.reuseModelBtn.setObjectName("reuseModelBtn")
        self.gridLayout_7.addWidget(self.reuseModelBtn, 0, 1, 1, 1)
        self.exportBtn = QtWidgets.QToolButton(self.groupBox_data_mgt)
        self.exportBtn.setObjectName("exportBtn")
        self.gridLayout_7.addWidget(self.exportBtn, 0, 4, 1, 1)
        self.gridLayout_5.addWidget(self.groupBox_data_mgt, 0, 0, 1, 2)
        self.groupBox2d = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox2d.setObjectName("groupBox2d")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.groupBox2d)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.surfListW = QtWidgets.QListWidget(self.groupBox2d)
        self.surfListW.setObjectName("surfListW")
        self.gridLayout_10.addWidget(self.surfListW, 0, 0, 1, 1)
        self.drawSurfButton = QtWidgets.QToolButton(self.groupBox2d)
        self.drawSurfButton.setObjectName("drawSurfButton")
        self.gridLayout_10.addWidget(self.drawSurfButton, 0, 2, 1, 1)
        self.gridLayout_5.addWidget(self.groupBox2d, 3, 0, 1, 2)
        self.groupBox_2 = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox_2.setObjectName("groupBox_2")
        self.formLayout = QtWidgets.QFormLayout(self.groupBox_2)
        self.formLayout.setObjectName("formLayout")
        self.deformationDistLabel = QtWidgets.QLabel(self.groupBox_2)
        self.deformationDistLabel.setObjectName("deformationDistLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.deformationDistLabel)
        self.deformationDistSpinBox = QtWidgets.QDoubleSpinBox(self.groupBox_2)
        self.deformationDistSpinBox.setSingleStep(0.1)
        self.deformationDistSpinBox.setProperty("value", 2.0)
        self.deformationDistSpinBox.setObjectName("deformationDistSpinBox")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.deformationDistSpinBox)
        self.gridLayout_5.addWidget(self.groupBox_2, 4, 0, 1, 2)
        self.tabWidget.addTab(self.tab_processing, "")
        self.horizontalLayout_2.addWidget(self.tabWidget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 668, 21))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionNew = QtWidgets.QAction(MainWindow)
        self.actionNew.setObjectName("actionNew")
        self.actionOpen_model = QtWidgets.QAction(MainWindow)
        self.actionOpen_model.setObjectName("actionOpen_model")
        self.actionNew_simulation = QtWidgets.QAction(MainWindow)
        self.actionNew_simulation.setObjectName("actionNew_simulation")
        self.iterationsLabel.setBuddy(self.iterationsSpinBox)
        self.simulationTimeLabel.setBuddy(self.simulationTimeLineEdit)
        self.saveAllAtomsPositionLabel.setBuddy(self.saveAllAtomsPositionCheckBox)
        self.tryToUseGPULabel.setBuddy(self.tryToUseGPUCheckBox)

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.newModelBtn, self.loadModelBtn)
        MainWindow.setTabOrder(self.loadModelBtn, self.saveModelBtn)
        MainWindow.setTabOrder(self.saveModelBtn, self.loadSimuBtn)
        MainWindow.setTabOrder(self.loadSimuBtn, self.paramsTreeWidget)
        MainWindow.setTabOrder(self.paramsTreeWidget, self.gotoSimuBtn)
        MainWindow.setTabOrder(self.gotoSimuBtn, self.iterationsSpinBox)
        MainWindow.setTabOrder(self.iterationsSpinBox, self.simulationTimeLineEdit)
        MainWindow.setTabOrder(self.simulationTimeLineEdit, self.saveAllAtomsPositionCheckBox)
        MainWindow.setTabOrder(self.saveAllAtomsPositionCheckBox, self.tryToUseGPUCheckBox)
        MainWindow.setTabOrder(self.tryToUseGPUCheckBox, self.simuBtn)
        MainWindow.setTabOrder(self.simuBtn, self.RTViewBtn)
        MainWindow.setTabOrder(self.RTViewBtn, self.gotoProcessBtn)
        MainWindow.setTabOrder(self.gotoProcessBtn, self.saveRModelBtn)
        MainWindow.setTabOrder(self.saveRModelBtn, self.reuseModelBtn)
        MainWindow.setTabOrder(self.reuseModelBtn, self.saveSimuBtn)
        MainWindow.setTabOrder(self.saveSimuBtn, self.PDFButton)
        MainWindow.setTabOrder(self.PDFButton, self.PDFNSpinBox)
        MainWindow.setTabOrder(self.PDFNSpinBox, self.PDFDistSpinBox)
        MainWindow.setTabOrder(self.PDFDistSpinBox, self.lineListW)
        MainWindow.setTabOrder(self.lineListW, self.lineComboW)
        MainWindow.setTabOrder(self.lineComboW, self.plotB)
        MainWindow.setTabOrder(self.plotB, self.list1DOptions)
        MainWindow.setTabOrder(self.list1DOptions, self.surfListW)
        MainWindow.setTabOrder(self.surfListW, self.drawSurfButton)
        MainWindow.setTabOrder(self.drawSurfButton, self.FPSSpinBox)
        MainWindow.setTabOrder(self.FPSSpinBox, self.stepsByFrameSpinBox)
        MainWindow.setTabOrder(self.stepsByFrameSpinBox, self.makeMovieBtn)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        self.groupBox.setTitle(_translate("MainWindow", "Current model"))
        self.paramsTreeWidget.headerItem().setText(0, _translate("MainWindow", "Property"))
        self.paramsTreeWidget.headerItem().setText(1, _translate("MainWindow", "Value"))
        self.paramsTreeWidget.headerItem().setText(2, _translate("MainWindow", "Unit"))
        self.loadSimuBtn.setText(_translate("MainWindow", "Load simulation history"))
        self.gotoSimuBtn.setText(_translate("MainWindow", "Simulation"))
        self.loadModelBtn.setText(_translate("MainWindow", "Load"))
        self.newModelBtn.setText(_translate("MainWindow", "New"))
        self.saveModelBtn.setText(_translate("MainWindow", "Save"))
        self.newSimuBtn.setText(_translate("MainWindow", "New simulation from this model"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_model), _translate("MainWindow", "Model"))
        self.progress_groupBox.setTitle(_translate("MainWindow", "Progress"))
        self.ETA.setText(_translate("MainWindow", "0"))
        self.label_3.setText(_translate("MainWindow", "ETA : "))
        self.simuProgressBar.setFormat(_translate("MainWindow", "%v/%m"))
        self.simuBtn.setText(_translate("MainWindow", "Launch simulation"))
        self.gotoProcessBtn.setText(_translate("MainWindow", "Process data"))
        self.iterationsLabel.setText(_translate("MainWindow", "Iterations"))
        self.simulationTimeLabel.setText(_translate("MainWindow", "Simulation time (s)"))
        self.saveAllAtomsPositionLabel.setText(_translate("MainWindow", "Save all atoms position"))
        self.tryToUseGPULabel.setText(_translate("MainWindow", "Try to use GPU"))
        self.temperature_groupBox.setTitle(_translate("MainWindow", "Model tweaking"))
        self.designTemperatureBtn.setText(_translate("MainWindow", "Design temperature evolution"))
        self.editModelBtn.setText(_translate("MainWindow", "Edit"))
        self.groupBox_3.setTitle(_translate("MainWindow", "Current state"))
        self.RTViewBtn.setText(_translate("MainWindow", "View current state"))
        self.currentIterationLabel.setText(_translate("MainWindow", "Current iteration : "))
        self.currentIteration.setText(_translate("MainWindow", "0"))
        self.label_4.setText(_translate("MainWindow", "Current time (s) : "))
        self.currentTime.setText(_translate("MainWindow", "0"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_simu), _translate("MainWindow", "Simulation"))
        self.groupBox1d.setTitle(_translate("MainWindow", "1D graphics"))
        self.asAFunctionOfLabel.setText(_translate("MainWindow", "as a function of"))
        self.plotB.setText(_translate("MainWindow", "Plot"))
        self.groupBox_1doptions.setTitle(_translate("MainWindow", "1D options"))
        self.groupBoxMovie.setTitle(_translate("MainWindow", "Movie"))
        self.makeMovieBtn.setText(_translate("MainWindow", "Make movie"))
        self.fPSLabel.setText(_translate("MainWindow", "FPS"))
        self.stepsByFrameLabel.setText(_translate("MainWindow", "Steps by frame"))
        self.groupBox1d_stat.setTitle(_translate("MainWindow", "Static 1D graphics"))
        self.PDFButton.setText(_translate("MainWindow", "Pair Distribution Function"))
        self.label.setText(_translate("MainWindow", "Number of atoms :"))
        self.label_2.setText(_translate("MainWindow", "Maximum distance (times r_cut) :"))
        self.groupBox_data_mgt.setTitle(_translate("MainWindow", "Data management"))
        self.saveSimuBtn.setText(_translate("MainWindow", "Save simulation history"))
        self.saveRModelBtn.setText(_translate("MainWindow", "Save resulting model"))
        self.reuseModelBtn.setText(_translate("MainWindow", "New simulation with resulting model"))
        self.exportBtn.setText(_translate("MainWindow", "Export to CSV"))
        self.groupBox2d.setTitle(_translate("MainWindow", "2D graphics"))
        self.drawSurfButton.setText(_translate("MainWindow", "Draw"))
        self.groupBox_2.setTitle(_translate("MainWindow", "2D parameters"))
        self.deformationDistLabel.setText(_translate("MainWindow", "Maximum distance for calculation (times r_cut)"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_processing), _translate("MainWindow", "Processing"))
        self.actionNew.setText(_translate("MainWindow", "New model"))
        self.actionOpen_model.setText(_translate("MainWindow", "Open model"))
        self.actionNew_simulation.setText(_translate("MainWindow", "New simulation"))

